@@ -240,7 +240,7 @@ fn emit_build_output(
 ///
 /// The construction includes:
 ///
-/// * Set environment varibles for the build script run.
+/// * Set environment variables for the build script run.
 /// * Create the output dir (`OUT_DIR`) for the build script output.
 /// * Determine if the build script needs a re-run.
 /// * Run the build script and store its output.
@@ -281,7 +281,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         .env("NUM_JOBS", &bcx.jobs().to_string())
         .env("TARGET", bcx.target_data.short_name(&unit.kind))
         .env("DEBUG", debug.to_string())
-        .env("OPT_LEVEL", &unit.profile.opt_level.to_string())
+        .env("OPT_LEVEL", &unit.profile.opt_level)
         .env(
             "PROFILE",
             match unit.profile.root {
@@ -299,17 +299,18 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         cmd.env(&var, value);
     }
 
-    if let Some(linker) = &bcx.target_data.target_config(unit.kind).linker {
-        cmd.env(
-            "RUSTC_LINKER",
-            linker.val.clone().resolve_program(bcx.config),
-        );
+    if let Some(linker) = &cx.compilation.target_linker(unit.kind) {
+        cmd.env("RUSTC_LINKER", linker);
     }
 
     if let Some(links) = unit.pkg.manifest().links() {
         cmd.env("CARGO_MANIFEST_LINKS", links);
     }
 
+    if let Some(trim_paths) = unit.profile.trim_paths.as_ref() {
+        cmd.env("CARGO_TRIM_PATHS", trim_paths.to_string());
+    }
+
     // Be sure to pass along all enabled features for this package, this is the
     // last piece of statically known information that we have.
     for feat in &unit.features {
@@ -356,6 +357,10 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
     );
     cmd.env_remove("RUSTFLAGS");
 
+    if cx.bcx.ws.config().extra_verbose() {
+        cmd.display_env_vars();
+    }
+
     // Gather the set of native dependencies that this package has along with
     // some other variables to close over.
     //
@@ -402,10 +407,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
     paths::create_dir_all(&script_out_dir)?;
 
     let nightly_features_allowed = cx.bcx.config.nightly_features_allowed;
-    let extra_check_cfg = match cx.bcx.config.cli_unstable().check_cfg {
-        Some((_, _, _, output)) => output,
-        None => false,
-    };
+    let extra_check_cfg = cx.bcx.config.cli_unstable().check_cfg;
     let targets: Vec<Target> = unit.pkg.targets().to_vec();
     // Need a separate copy for the fresh closure.
     let targets_fresh = targets.clone();
@@ -446,7 +448,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
                     ))
                 })?;
                 let data = &script_output.metadata;
-                for &(ref key, ref value) in data.iter() {
+                for (key, value) in data.iter() {
                     cmd.env(
                         &format!("DEP_{}_{}", super::envify(&name), super::envify(key)),
                         value,
@@ -684,22 +686,24 @@ impl BuildOutput {
                 Ok(line) => line.trim(),
                 Err(..) => continue,
             };
-            let mut iter = line.splitn(2, ':');
-            if iter.next() != Some("cargo") {
-                // skip this line since it doesn't start with "cargo:"
-                continue;
-            }
-            let data = match iter.next() {
-                Some(val) => val,
-                None => continue,
+            let data = match line.split_once(':') {
+                Some(("cargo", val)) => {
+                    if val.starts_with(":") {
+                        // Line started with `cargo::`.
+                        bail!("unsupported output in {whence}: `{line}`\n\
+                            Found a `cargo::key=value` build directive which is reserved for future use.\n\
+                            Either change the directive to `cargo:key=value` syntax (note the single `:`) or upgrade your version of Rust.\n\
+                            See https://doc.rust-lang.org/cargo/reference/build-scripts.html#outputs-of-the-build-script \
+                            for more information about build script outputs.");
+                    }
+                    val
+                }
+                _ => continue,
             };
 
             // getting the `key=value` part of the line
-            let mut iter = data.splitn(2, '=');
-            let key = iter.next();
-            let value = iter.next();
-            let (key, value) = match (key, value) {
-                (Some(a), Some(b)) => (a, b.trim_end()),
+            let (key, value) = match data.split_once('=') {
+                Some((a,b)) => (a, b.trim_end()),
                 // Line started with `cargo:` but didn't match `key=value`.
                 _ => bail!("invalid output in {}: `{}`\n\
                     Expected a line with `cargo:key=value` with an `=` character, \
@@ -758,9 +762,7 @@ impl BuildOutput {
                     check_and_add_target!("bin", Target::is_bin, LinkArgTarget::Bin);
                 }
                 "rustc-link-arg-bin" => {
-                    let mut parts = value.splitn(2, '=');
-                    let bin_name = parts.next().unwrap().to_string();
-                    let arg = parts.next().ok_or_else(|| {
+                    let (bin_name, arg) = value.split_once('=').ok_or_else(|| {
                         anyhow::format_err!(
                             "invalid instruction `cargo:{}={}` from {}\n\
                                 The instruction should have the form cargo:{}=BIN=ARG",
@@ -783,7 +785,10 @@ impl BuildOutput {
                             bin_name
                         );
                     }
-                    linker_args.push((LinkArgTarget::SingleBin(bin_name), arg.to_string()));
+                    linker_args.push((
+                        LinkArgTarget::SingleBin(bin_name.to_owned()),
+                        arg.to_string(),
+                    ));
                 }
                 "rustc-link-arg-tests" => {
                     check_and_add_target!("test", Target::is_test, LinkArgTarget::Test);
@@ -802,7 +807,7 @@ impl BuildOutput {
                     if extra_check_cfg {
                         check_cfgs.push(value.to_string());
                     } else {
-                        warnings.push(format!("cargo:{} requires -Zcheck-cfg=output flag", key));
+                        warnings.push(format!("cargo:{} requires -Zcheck-cfg flag", key));
                     }
                 }
                 "rustc-env" => {
@@ -827,7 +832,7 @@ impl BuildOutput {
                                 None => return false,
                                 Some(n) => n,
                             };
-                            // ALLOWED: the process of rustc boostrapping reads this through
+                            // ALLOWED: the process of rustc bootstrapping reads this through
                             // `std::env`. We should make the behavior consistent. Also, we
                             // don't advertise this for bypassing nightly.
                             #[allow(clippy::disallowed_methods)]
@@ -929,12 +934,9 @@ impl BuildOutput {
     ///
     /// [`cargo:rustc-env`]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html#rustc-env
     pub fn parse_rustc_env(value: &str, whence: &str) -> CargoResult<(String, String)> {
-        let mut iter = value.splitn(2, '=');
-        let name = iter.next();
-        let val = iter.next();
-        match (name, val) {
-            (Some(n), Some(v)) => Ok((n.to_owned(), v.to_owned())),
-            _ => bail!("Variable rustc-env has no value in {}: {}", whence, value),
+        match value.split_once('=') {
+            Some((n, v)) => Ok((n.to_owned(), v.to_owned())),
+            _ => bail!("Variable rustc-env has no value in {whence}: {value}"),
         }
     }
 }
@@ -1125,10 +1127,7 @@ fn prev_build_output(cx: &mut Context<'_, '_>, unit: &Unit) -> (Option<BuildOutp
             &unit.pkg.to_string(),
             &prev_script_out_dir,
             &script_out_dir,
-            match cx.bcx.config.cli_unstable().check_cfg {
-                Some((_, _, _, output)) => output,
-                None => false,
-            },
+            cx.bcx.config.cli_unstable().check_cfg,
             cx.bcx.config.nightly_features_allowed,
             unit.pkg.targets(),
         )
