@@ -684,8 +684,7 @@ fn infer_builtin_macros_line() {
         }
         "#,
         expect![[r#"
-            !0..1 '0': i32
-            !0..6 '0asu32': u32
+            !0..4 '0u32': u32
             63..87 '{     ...!(); }': ()
             73..74 'x': u32
         "#]],
@@ -723,8 +722,7 @@ fn infer_builtin_macros_column() {
         }
         "#,
         expect![[r#"
-            !0..1 '0': i32
-            !0..6 '0asu32': u32
+            !0..4 '0u32': u32
             65..91 '{     ...!(); }': ()
             75..76 'x': u32
         "#]],
