@@ -37,8 +37,11 @@ use crate::{
 // ```
 pub(crate) fn qualify_path(acc: &mut Assists, ctx: &AssistContext<'_>) -> Option<()> {
     let (import_assets, syntax_under_caret) = find_importable_node(ctx)?;
-    let mut proposed_imports =
-        import_assets.search_for_relative_paths(&ctx.sema, ctx.config.prefer_no_std);
+    let mut proposed_imports = import_assets.search_for_relative_paths(
+        &ctx.sema,
+        ctx.config.prefer_no_std,
+        ctx.config.prefer_prelude,
+    );
     if proposed_imports.is_empty() {
         return None;
     }
